﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Windows.Threading;

namespace Spiel
{
    public partial class MainWindow : Window
    {
        DispatcherTimer timer = new DispatcherTimer();
        List<Spielobjekt> alleObjekte = new List<Spielobjekt>();
        Raumschiff raumschiff;
        List<Photonentorpedo> photonentorpedos = new List<Photonentorpedo>();
        List<Asteroid> asteroiden = new List<Asteroid>();

        public MainWindow()
        {
            InitializeComponent();

            Spielobjekt.SetzeAbmessungenSpielfeld(zeichenfläche.Width, zeichenfläche.Height);
            raumschiff = new Raumschiff();
            alleObjekte.Add(raumschiff);

            timer.Tick += Animiere;
            timer.Interval = TimeSpan.FromSeconds(0.1);
            timer.IsEnabled = true;
        }

        void Animiere(object sender, EventArgs e)
        {
            if(asteroiden.Count < 5)
            {
                Asteroid a = Asteroid.ErzeugeAmRand();
                alleObjekte.Add(a);
                asteroiden.Add(a);
            }

            // alles bewegen
            int i = 0;
            while(i < alleObjekte.Count)
            {
                Spielobjekt o = alleObjekte[i];
                o.Bewege(timer.Interval.TotalSeconds);
                if(o is Photonentorpedo)
                {
                    Photonentorpedo pt = (Photonentorpedo)o;
                    if(pt.HatSpielfeldVerlassen())
                    {
                        alleObjekte.RemoveAt(i);
                        photonentorpedos.Remove(pt);
                        continue; // gleich nochmal bei Index i prüfen, weil die nachfolgenden aufrücken
                    }
                }
                else
                {
                    o.BringeAufZeichenflächeZurück();
                }
                i++;
            }

            // Asteroiden von Photonentorpedos getroffen?
            i = 0;
            while(i < asteroiden.Count)
            {
                bool getroffen = false;
                int j = 0;
                for (; j < photonentorpedos.Count; j++)
                {
                    Photonentorpedo pt = photonentorpedos[j];
                    if (asteroiden[i].EnthältPunkt(pt.X, pt.Y))
                    {
                        getroffen = true;
                        break;
                    }
                }
                if(getroffen)
                {
                    alleObjekte.Remove(photonentorpedos[j]);
                    photonentorpedos.RemoveAt(j);
                    alleObjekte.Remove(asteroiden[i]);
                    asteroiden.RemoveAt(i);
                    continue; // gleich nochmal bei Index i prüfen, weil die nachfolgenden aufrücken
                }
                i++;
            }

            zeichenfläche.Children.Clear();
            foreach (var o in alleObjekte)
            {
                o.Zeichne(zeichenfläche);
            }

            bool raumschiffGetroffen = false;
            foreach (var a in asteroiden)
            {
                if(a.EnthältPunkt(raumschiff.X, raumschiff.Y))
                {
                    raumschiffGetroffen = true;
                    break;
                }
            }
            if(raumschiffGetroffen)
            {
                MessageBox.Show("Game over!");
                asteroiden.Clear();
                photonentorpedos.Clear();
                alleObjekte.Clear();
                raumschiff = new Raumschiff();
                alleObjekte.Add(raumschiff);
            }
        }

        private void Window_KeyDown(object sender, KeyEventArgs e)
        {
            switch(e.Key)
            {
                case Key.Right:
                    raumschiff.Drehe(true);
                    break;
                case Key.Left:
                    raumschiff.Drehe(false);
                    break;
                case Key.Up:
                    raumschiff.Beschleunige(true);
                    break;
                case Key.Down:
                    raumschiff.Beschleunige(false);
                    break;
                case Key.Space:
                    Photonentorpedo pt = new Photonentorpedo(raumschiff.X, raumschiff.Y, raumschiff.Vx, raumschiff.Vy, raumschiff.Winkel);
                    alleObjekte.Add(pt);
                    photonentorpedos.Add(pt);
                    break;
                default:
                    break;
            }
        }
    }
}
